@@ -1,6 +1,10 @@
 import datetime
 import collections
 
+import hashlib
+import base64
+import hmac
+
 import pytz
 from future.standard_library import hooks
 
@@ -9,7 +13,7 @@ from pycronofy.auth import Auth
 from pycronofy.batch import BatchEntry
 from pycronofy.batch import BatchResponse
 from pycronofy.datetime_utils import format_event_time
-from pycronofy.exceptions import PyCronofyPartialSuccessError, PyCronofyRequestError
+from pycronofy.exceptions import PyCronofyPartialSuccessError, PyCronofyRequestError, PyCronofyValidationError
 from pycronofy.pagination import Pages
 from pycronofy.request_handler import RequestHandler
 from pycronofy.validation import validate
@@ -645,6 +649,48 @@ class Client(object):
 
         return self.request_handler.post(endpoint='real_time_scheduling', data=args, use_api_key=True).json()
 
+    def get_real_time_scheduling_status(self, token=None, real_time_scheduling_id=None):
+        """ Gets the status of a Real-Time Scheduling link by either ID or its URL token.
+
+        :param string token: the link's token from its URL
+        :param string real_time_scheduling_id: the ID of the link
+        :return: Dictionary containing the current status of the Real-Time Scheduling link
+        :rtype: ``dict``
+        """
+        if real_time_scheduling_id and token:
+            raise PyCronofyValidationError('Must pass one of token or real_time_scheduling_id.', 'get_real_time_scheduling_status')
+        elif real_time_scheduling_id:
+            return self.request_handler.get(
+                endpoint='real_time_scheduling/%s' % real_time_scheduling_id,
+                use_api_key=True
+            ).json()
+        elif token:
+            return self.request_handler.get(
+                endpoint='real_time_scheduling',
+                params={
+                    'token': token
+                },
+                use_api_key=True
+            ).json()
+        else:
+            raise PyCronofyValidationError('Must pass either token or real_time_scheduling_id.', 'get_real_time_scheduling_status')
+
+    def disable_real_time_scheduling_link(self, real_time_scheduling_id, display_message):
+        """ Disables a Real-Time Scheduling link, with a display message for visitors to the URL
+
+        :param string real_time_scheduling_id: the ID of the link
+        :param string display_message: a message to show visitors to the disabled link (<500 characters)
+        :return: Dictionary containing the current status of the Real-Time Scheduling link
+        :rtype: ``dict``
+        """
+        return self.request_handler.post(
+            endpoint='real_time_scheduling/%s/disable' % real_time_scheduling_id,
+            data={
+                'display_message': display_message
+            },
+            use_api_key=True
+        ).json()
+
     def real_time_sequencing(self, availability, oauth, event, target_calendars=(), minimum_notice=None):
         """Generates an real time sequencing link to start the OAuth process with
         an event to be automatically upserted
@@ -897,3 +943,38 @@ class Client(object):
         :param string availability_rule_id: ID of the Availability Rule to delete.
         """
         self.request_handler.delete(endpoint='availability_rules/%s' % availability_rule_id)
+
+    def hmac_valid(self, hmac_string, body):
+        """Verifies a HMAC from a push notification using the client secret.
+
+        :hmac_string: A String containing comma-separated values describing HMACs of the notification taken from the
+                        Cronofy-HMAC-SHA256 header.
+        :body: A String of the body of the notification.
+        :return: TRUE if one of the HMAC provided matches the one calculated using the client secret, otherwise FALSE.
+        """
+        if hmac_string is None or not hmac_string:
+            return False
+
+        # In order to support Python 2.7 we generate the hmac and calculate the digest in two steps using `new` and `digest`
+        generated = hmac.new(self.auth.client_secret.encode(), body.encode(), hashlib.sha256)
+        digest = generated.digest()
+        calculated = base64.b64encode(digest).strip()
+
+        hmac_list = hmac_string.split(',')
+
+        # compare_digest used to reduce vulnerability to timing attacks
+        return any(hmac.compare_digest(value.encode(), calculated) for value in hmac_list)
+
+    def get_conferencing_services_auth_link(self, redirect_uri, provider_name=None):
+        """Get a URL to direct the user to so they can authorize with a conferencing provider
+
+        :param string redirect_uri: URL to redirect the user to after authorization.
+        :param string provider_name: Optional provider identifier to pre-select for the user.
+         Refer to the documentation at https://docs.cronofy.com/developers/api/conferencing-services/authorization for possible values.
+        :return: the URL to direct the user to
+        """
+        data = {"redirect_uri": redirect_uri}
+        if provider_name:
+            data["provider_name"] = provider_name
+        response = self.request_handler.post(endpoint="conferencing_service_authorizations", data=data)
+        return response.json()["authorization_request"]["url"]
